package de.kif.frontend.views.calendar

import de.kif.frontend.iterator
import de.kif.frontend.launch
import de.kif.frontend.repository.ScheduleRepository
import de.westermann.kwebview.View
import org.w3c.dom.HTMLElement
import org.w3c.dom.get
import kotlin.browser.document


class Calendar(calendar: HTMLElement): View(calendar) {
    var calendarEntries: List<CalendarEntry> = emptyList()
    var calendarCells: List<CalendarCell> = emptyList()

    val day: Int

    init {
        val editable = calendar.dataset["editable"]?.toBoolean() ?: false
        day = calendar.dataset["day"]?.toIntOrNull() ?: -1

        calendarEntries = document.getElementsByClassName("calendar-entry")
            .iterator().asSequence().map{ CalendarEntry(this, it) }.onEach { it.editable = editable }.toList()

        calendarCells = document.getElementsByClassName("calendar-cell")
            .iterator().asSequence().filter { it.dataset["time"] != null }.map(::CalendarCell).toList()

        if (editable) {
            CalendarEdit(this, calendar.querySelector(".calendar-edit") as HTMLElement)
        }

        ScheduleRepository.onCreate {
            launch {
                val schedule = ScheduleRepository.get(it) ?: throw  NoSuchElementException()
                calendarEntries += CalendarEntry.create(this, schedule).also { it.editable = editable }
            }
        }
        ScheduleRepository.onUpdate {
            launch {
                val schedule = ScheduleRepository.get(it) ?: throw  NoSuchElementException()
                var found = false
                for (entry in calendarEntries) {
                    if (entry.scheduleId == it) {
                        entry.load(schedule)
                        found = true
                    }
                }
                if (!found) {
                    calendarEntries += CalendarEntry.create(this, schedule).also { it.editable = editable }
                }
            }
        }
        ScheduleRepository.onDelete {
            for (entry in calendarEntries) {
                if (entry.scheduleId == it) {
                    entry.html.remove()
                }
            }
        }
    }
}

fun initCalendar() {
    Calendar(document.getElementsByClassName("calendar")[0] as? HTMLElement ?: return)
}